@@ -12,7 +12,11 @@ def get_aggregator() -> Aggregator:
     return _server
 
 
-def _create_aggregator(dispatcher: AggregatorDispatcher, publisher: FrontendPublisher, webpush_publisher: WebPushPublisher, secret: str) -> Aggregator:
+def _create_aggregator(
+        dispatcher: AggregatorDispatcher,
+        publisher: FrontendPublisher,
+        webpush_publisher: WebPushPublisher,
+        secret: str) -> Aggregator:
     global _server
     if _server is not None:
         return _server
