@@ -2,10 +2,12 @@ from __future__ import annotations
 
 import itertools
 import logging
+from threading import Lock
 import uuid
-from queue import Empty, Queue
-from typing import Iterable, Literal, Set
+from queue import Queue
+from typing import Iterable, Literal
 
+from openpectus.engine.command_manager import CommandManager
 import openpectus.protocol.models as Mdl
 from openpectus.engine.archiver import ArchiverTag
 from openpectus.engine.hardware import HardwareLayerException, RegisterDirection
@@ -16,7 +18,7 @@ from openpectus.lang.exec.base_unit import BaseUnitProvider
 from openpectus.lang.exec.clock import Clock, WallClock
 from openpectus.lang.exec.commands import CommandRequest
 from openpectus.lang.exec.errors import (
-    EngineError, InterpretationError, InterpretationInternalError, MethodEditError
+    EngineError, EngineNotInitializedError, InterpretationError, InterpretationInternalError, MethodEditError
 )
 from openpectus.lang.exec.events import EventEmitter
 from openpectus.lang.exec.pinterpreter import PInterpreter, InterpreterContext, Tracking
@@ -30,7 +32,7 @@ from openpectus.lang.exec.tags import (
 )
 from openpectus.lang.exec.tags_impl import BlockTimeTag, MarkTag, ScopeTimeTag
 from openpectus.lang.exec.timer import EngineTimer, OneThreadTimer
-from openpectus.lang.exec.uod import UnitOperationDefinitionBase, UodCommand
+from openpectus.lang.exec.uod import UnitOperationDefinitionBase
 
 logger = logging.getLogger(__name__)
 frontend_logger = logging.getLogger(__name__ + ".frontend")
@@ -118,10 +120,6 @@ class Engine(InterpreterContext):
 
         self.uod.system_tags = self._system_tags
 
-        self.cmd_queue: Queue[CommandRequest] = Queue()
-        """ Commands to execute, coming from interpreter and from aggregator """
-        self.cmd_executing: list[CommandRequest] = []
-        """ Uod commands currently being excuted """
         self.tag_updates: Queue[Tag] = Queue()
         """ Tags updated in last tick """
 
@@ -145,9 +143,6 @@ class Engine(InterpreterContext):
         self._last_error: Exception | None = None
         """ Cause of error_state"""
 
-        self._method_manager: MethodManager = MethodManager(uod.get_command_names(), self)
-        """ The model handling changes to method code and interpreter running it """
-
         # initialize state
         self.uod.tags.add_listener(self._uod_listener)
         self._system_tags.add_listener(self._system_listener)
@@ -155,6 +150,23 @@ class Engine(InterpreterContext):
         self._emitter = EventEmitter(self._tags)
         self._tick_timer.set_tick_fn(self.tick)
 
+        # there attributes must be declared before self.on_interpreter_reset() can use them
+        self._interpreter: PInterpreter | None = None
+        self._tracking: Tracking | None = None
+        self._command_manager: CommandManager | None = None
+
+        self._method_manager: MethodManager = MethodManager(uod.get_command_names(), self, self.on_interpreter_reset)
+        """ The model handling changes to method code and interpreter running it """
+
+        self._lock = Lock()
+
+
+    def on_interpreter_reset(self, interpreter: PInterpreter):
+        self._interpreter = interpreter
+        self._tracking = interpreter.tracking
+        restart_request_pending = None if self._command_manager is None else self._command_manager.restart_request_pending
+        self._command_manager = CommandManager(self._tracking, self.uod, self.registry, restart_request_pending)
+
     def __str__(self) -> str:
         return (f'{self.__class__.__name__}(uod={self.uod}, is_running={self.is_running}, ' +
                 f'has_error_state={self.has_error_state})')
@@ -183,11 +195,15 @@ class Engine(InterpreterContext):
 
     @property
     def interpreter(self) -> PInterpreter:
-        return self.method_manager.interpreter
+        if self._interpreter is None:
+            raise EngineNotInitializedError("interpreter not set")
+        return self._interpreter
 
     @property
     def tracking(self) -> Tracking:
-        return self.interpreter.tracking
+        if self._tracking is None:
+            raise EngineNotInitializedError("tracking not set")
+        return self._tracking
 
     def cleanup(self):
         self.emitter.emit_on_engine_shutdown()
@@ -250,45 +266,52 @@ class Engine(InterpreterContext):
         self.read_process_image()
 
         # execute phase
-        # excecute interpreter tick
-        if self._runstate_started and \
-                not self._runstate_paused and \
-                not self._runstate_holding and \
-                not self._runstate_stopping:
+        with self._lock:
+            self.tracking.tick(tick_time, self._tick_number)
+
+            # excecute interpreter tick
+            if self._runstate_started and \
+                    not self._runstate_paused and \
+                    not self._runstate_holding and \
+                    not self._runstate_stopping:
+                try:
+                    # run one tick of interpretation, i.e. one instruction
+                    self.interpreter.tick(tick_time, self._tick_number)
+                except InterpretationInternalError as ex:
+                    logger.fatal("A serious internal interpreter error occured. The method should be stopped. If it is resumed, \
+                                additional errors may occur.", exc_info=True)
+                    self.set_error_state(ex)
+                except EngineError as eex:
+                    logger.error(eex.message)
+                    if eex.user_message is not None:
+                        frontend_logger.error(eex.user_message)
+                    self.set_error_state(eex)
+                except InterpretationError as ie:
+                    logger.error("Interpretation error", exc_info=True)
+                    if ie.user_message is not None:
+                        frontend_logger.error(ie.user_message)
+                    self.set_error_state(ie)
+                except Exception as ex:
+                    logger.error("Unhandled interpretation error", exc_info=True)
+                    frontend_logger.error("Method error")
+                    self.set_error_state(ex)
+
+            # update calculated tags
+            if self._runstate_started:
+                self.update_calculated_tags(tick_time, increment_time)
+
+            # execute queued commands, go to error_state on error
             try:
-                # run one tick of interpretation, i.e. one instruction
-                self.interpreter.tick(tick_time, self._tick_number)
-            except InterpretationInternalError as ex:
-                logger.fatal("A serious internal interpreter error occured. The method should be stopped. If it is resumed, \
-                             additional errors may occur.", exc_info=True)
-                self.set_error_state(ex)
-            except EngineError as eex:
-                logger.error(eex.message)
-                if eex.user_message is not None:
-                    frontend_logger.error(eex.user_message)
-                self.set_error_state(eex)
-            except InterpretationError as ie:
-                logger.error("Interpretation error", exc_info=True)
-                if ie.user_message is not None:
-                    frontend_logger.error(ie.user_message)
-                self.set_error_state(ie)
+                assert self._command_manager is not None
+                self._command_manager.tick(tick_time, self._tick_number)
             except Exception as ex:
-                logger.error("Unhandled interpretation error", exc_info=True)
-                frontend_logger.error("Method error")
                 self.set_error_state(ex)
 
-        # update calculated tags
-        if self._runstate_started:
-            self.update_calculated_tags(tick_time, increment_time)
+            # notify of tag changes
+            self.notify_tag_updates()
 
-        # execute queued commands, go to error_state on error
-        self.execute_commands()
-
-        # notify of tag changes
-        self.notify_tag_updates()
-
-        # write phase, error_state on HardwareLayerException
-        self.write_process_image()
+            # write phase, error_state on HardwareLayerException
+            self.write_process_image()
 
     def read_process_image(self):
         """ Read data from relevant hw registers into tags"""
@@ -320,123 +343,19 @@ class Engine(InterpreterContext):
 
         # Process Time  - 0 at start, increments when System State is Run
         process_time = self._system_tags[SystemTagName.PROCESS_TIME]
-        process_time_value = process_time.as_number()
+        process_time_value = process_time.as_float()
         if sys_state.get_value() == SystemStateEnum.Running:
             process_time.set_value(process_time_value + increment_time, tick_time)
 
         # Run Time      - 0 at start, increments when System State is not Stopped
         run_time = self._system_tags[SystemTagName.RUN_TIME]
-        run_time_value = run_time.as_number()
+        run_time_value = run_time.as_float()
         if sys_state.get_value() not in [SystemStateEnum.Stopped, SystemStateEnum.Restarting]:
             run_time.set_value(run_time_value + increment_time, tick_time)
 
         # Execute the tick lifetime hook on tags
         self.emitter.emit_on_tick(tick_time, increment_time)
 
-    def execute_commands(self):
-        done = False
-        # add command request from incoming queue
-
-        while self.cmd_queue.qsize() > 0 and not done:
-            try:
-                engine_command = self.cmd_queue.get()
-                # Note: New commands are inserted at the beginning of the list.
-                # This allows simpler cancellation of identical/overlapping commands
-                self.cmd_executing.insert(0, engine_command)
-            except Empty:
-                done = True
-
-        # execute a tick of each running command
-        cmds_done: Set[CommandRequest] = set()
-        latest_cmd = "(none)"
-        try:
-            for c in self.cmd_executing:
-                latest_cmd = c.name
-                if c not in cmds_done:
-                    # Note: Executing one command may cause other commands to be cancelled (by identical or overlapping
-                    # commands) Rather than modify self.cmd_executing (while iterating over it), cancelled/completed
-                    # commands are added to the cmds_done set.
-                    self._execute_command(c, cmds_done)
-        except ValueError as ve:
-            logger.error(f"Error executing command: '{latest_cmd}'. Command failed with error: {ve}", exc_info=True)
-            frontend_logger.error(f"Command '{latest_cmd}' failed: {ve}")
-            self.set_error_state(ve)
-        except Exception as ex:
-            logger.error(f"Error executing command: '{latest_cmd}'", exc_info=True)
-            frontend_logger.error(f"Error executing command: '{latest_cmd}'")
-            self.set_error_state(ex)
-        finally:
-            for c_done in cmds_done:
-                self.cmd_executing.remove(c_done)
-
-    def _execute_command(self, cmd_request: CommandRequest, cmds_done: Set[CommandRequest]):
-        # execute an internal engine command or a uod command
-
-        logger.debug("Executing command: " + cmd_request.name)
-        if cmd_request.name is None or len(cmd_request.name.strip()) == 0:
-            logger.error("Command name empty")
-            frontend_logger.error("Cannot execute command with empty name")
-            cmds_done.add(cmd_request)
-            return
-
-        try:
-            if EngineCommandEnum.has_value(cmd_request.name):
-                self._execute_internal_command(cmd_request, cmds_done)
-            else:
-                self._execute_uod_command(cmd_request, cmds_done)
-        except Exception:
-            self.tracking.mark_failed(cmd_request)
-            logger.error("Error running command " + cmd_request.name, exc_info=True)
-            raise
-
-    def _execute_internal_command(self, cmd_request: CommandRequest, cmds_done: Set[CommandRequest]):  # noqa C901
-        if not self._runstate_started and cmd_request.name not in [EngineCommandEnum.START, EngineCommandEnum.RESTART]:
-            logger.warning(f"Command {cmd_request.name} is invalid when Engine is not running")
-            cmds_done.add(cmd_request)
-            return
-
-        # an existing, long running engine_command is running. other commands must wait
-        # Note: we need a priority mechanism - even Stop is waiting here
-        command = self.registry.get_running_internal_command()
-        if command is not None:
-            if cmd_request.name == command.name:
-                if not command.is_finalized():
-                    command.tick()
-                if command.has_failed():
-                    cmds_done.add(cmd_request)
-                    self.tracking.mark_failed(cmd_request)
-                elif command.is_finalized():
-                    cmds_done.add(cmd_request)
-                    self.tracking.mark_completed(cmd_request)
-            return
-
-        # no engine command is running - start one
-        try:
-            command = self.registry.create_internal_command(cmd_request.name, cmd_request.instance_id)
-            args = cmd_request.arguments
-            if args is not None:
-                try:
-                    command.validate_arguments(args)
-                    logger.debug(f"Initialized command {cmd_request.name} with arguments '{args}'")
-                except Exception:
-                    raise EngineError(
-                        f"Failed to initialize arguments '{args}' for command '{cmd_request.name}'",
-                        "same"
-                    )
-        except ValueError:
-            raise EngineError(
-                f"Unknown internal engine command '{cmd_request.name}'",
-                f"Unknown command '{cmd_request.name}'")
-
-        self.tracking.mark_internal_command_started(command)
-        command.tick()
-        if command.has_failed():
-            self.tracking.mark_failed(command)
-            cmds_done.add(cmd_request)
-        elif command.is_finalized():
-            cmds_done.add(cmd_request)
-            self.tracking.mark_completed(command)
-
     def set_run_id(self) -> str:
         """ Creates a new run_id, sets the Run Id tag to it and returns it. """
         run_id = str(uuid.uuid4())
@@ -458,140 +377,6 @@ class Engine(InterpreterContext):
         # sense to wait until start
         self.method_manager.reset_interpreter()
 
-    def _cancel_uod_commands(self):
-        logger.debug("Cancelling all uod commands")
-        cmds_to_cancel: list[UodCommand] = []
-        for name, command in self.uod.command_instances.items():
-            if command.is_cancelled() or command.is_execution_complete() or command.is_finalized():
-                logger.debug(f"Skipping command '{name}' that is no longer running")
-            else:
-                cmds_to_cancel.append(command)
-
-        # call outside the loop because cancel modifies the collection
-        for command in cmds_to_cancel:
-            command.cancel()
-
-        if any(cmds_to_cancel):
-            cmd_names = ",".join([c.name for c in cmds_to_cancel])
-            logger.debug(f"Cancelled {len(cmds_to_cancel)} uod commands: {cmd_names}")
-
-    def _finalize_uod_commands(self):
-        logger.debug("Finalizing uod commands")
-        cmds_to_finalize: list[UodCommand] = []
-        for command in self.uod.command_instances.values():
-            if not command.is_finalized():
-                cmds_to_finalize.append(command)
-
-        # call outside the loop because finalize modifies the collection
-        for command in cmds_to_finalize:
-            command.finalize()
-
-    def _execute_uod_command(self, cmd_request: CommandRequest, cmds_done: Set[CommandRequest]):  # noqa C901
-        cmd_name = cmd_request.name
-        assert self.uod.has_command_name(cmd_name), f"Expected Uod to have command named '{cmd_name}'"
-
-        if self._runstate_stopping:
-            logger.debug(f"Skipping uod command '{cmd_name}', run is restarting")
-            return
-
-        if not self.uod.hwl.is_connected:
-            raise EngineError(
-                f"The hardware is disconnected. The command '{cmd_name}' was not allowed to start.",
-                "same")
-
-        cancel_this = False
-
-        # cancel any existing instance with same name
-        for c in [_c for _c in self.cmd_executing if _c not in cmds_done]:
-            if c.name == cmd_name and not c == cmd_request:
-                cmds_done.add(c)
-                command = self.tracking.get_command(c.instance_id)
-                if command is not None:
-                    command.cancel()
-                    command.finalize()
-                    self.tracking.mark_cancelled(c)
-                    logger.debug(f"Running command {c.name} cancelled because another was started")
-                else:
-                    logger.error(f"Cannot cancel command {c}. No runtime record found  for command: '{c.name=}' " +
-                                 f" and {c.instance_id=}")
-
-        # cancel any overlapping instance
-        for c in [_c for _c in self.cmd_executing if _c not in cmds_done]:
-            if not c == cmd_request:
-                for overlap_list in self.uod.overlapping_command_names_lists:
-                    if c.name in overlap_list and cmd_name in overlap_list:
-                        cmds_done.add(c)
-                        command = self.tracking.get_command(c.instance_id)
-                        if command is not None:
-                            command.cancel()
-                            command.finalize()
-                            self.tracking.mark_cancelled(c)
-                            logger.info(
-                                f"Running command {c.name} cancelled because overlapping command " +
-                                f"'{cmd_name}' was started")
-                        else:
-                            logger.error(f"Cannot cancel command {c}. No runtime record found for command: '{c.name=}' " +
-                                         f" and {c.instance_id=}")
-
-        if cancel_this:
-            # don't start command again that was just cancelled
-            return
-
-        # create or get command instance
-        if not self.uod.has_command_instance(cmd_name):
-            uod_command = self.uod.create_command(cmd_name, cmd_request.instance_id)
-        else:
-            uod_command = self.uod.get_command(cmd_name)
-
-        assert uod_command is not None, f"Failed to get uod_command for command '{cmd_name}'"
-
-        logger.debug(f"Parsing arguments '{cmd_request.arguments}' for uod command {cmd_name}")
-        parsed_args = uod_command.parse_args(cmd_request.arguments)
-
-        if parsed_args is None:
-            logger.error(f"Invalid argument string: '{cmd_request.arguments}' for command '{cmd_name}'")
-            cmds_done.add(cmd_request)
-            raise ValueError(f"Invalid arguments for command '{cmd_name}'")
-
-        # execute command state flow
-        try:
-            logger.debug(
-                f"Executing uod command: '{cmd_request.name}' with parsed args '{parsed_args}', " +
-                f"iteration {uod_command._exec_iterations}")
-            if uod_command.is_cancelled():
-                if not uod_command.is_finalized():
-                    cmds_done.add(cmd_request)
-                    uod_command.finalize()
-
-            if not uod_command.is_initialized():
-                uod_command.initialize()
-                logger.debug(f"Command {cmd_request.name} initialized")
-
-            if not uod_command.is_execution_started():
-                self.tracking.mark_uod_command_started(uod_command)
-                uod_command.execute(parsed_args)
-                logger.debug(f"Command {cmd_request.name} executed first iteration {uod_command._exec_iterations}")
-            elif not uod_command.is_execution_complete():
-                uod_command.execute(parsed_args)
-                logger.debug(f"Command {cmd_request.name} executed another iteration {uod_command._exec_iterations}")
-
-            if uod_command.is_execution_complete() and not uod_command.is_finalized():
-                self.tracking.mark_completed(cmd_request)
-                cmds_done.add(cmd_request)
-                uod_command.finalize()
-                logger.debug(f"Command {cmd_request.name} finalized")
-
-        except Exception:
-            # handle error locally because we need specific command cleanup
-            if not uod_command.is_cancelled():
-                uod_command.cancel()
-                self.tracking.mark_cancelled(uod_command)
-
-                logger.info(f"Cleaned up failed command {cmd_name}")
-
-            logger.error(f"Uod command execution failed. Command: '{cmd_request}'", exc_info=True)
-            raise
-
     def _apply_safe_state(self) -> TagValueCollection:
         current_values: list[TagValue] = []
         hwl = self.uod.hwl
@@ -676,11 +461,12 @@ class Engine(InterpreterContext):
 
     def schedule_execution(self, name: str, arguments: str = "", instance_id: str | None = None):
         """ Execute named command from interpreter """
+        assert self._command_manager is not None
         if instance_id is None:
             instance_id = self.tracking.create_instance_id(name)
         if EngineCommandEnum.has_value(name) or self.uod.has_command_name(name):
             request = CommandRequest.from_interpreter(name, arguments, instance_id)
-            self.cmd_queue.put_nowait(request)
+            self._command_manager.schedule(request)
         else:
             raise EngineError(
                 f"Invalid command type scheduled: '{name}'",
@@ -724,10 +510,11 @@ class Engine(InterpreterContext):
 
     def execute_control_command_from_user(self, name: str):
         """ Execute named command from user """
+        assert self._command_manager is not None
         if EngineCommandEnum.has_value(name) or self.uod.has_command_name(name):
             self._validate_control_command(name)
             request = CommandRequest.from_user(name, "", self.tracking.create_instance_id(name))
-            self.cmd_queue.put_nowait(request)
+            self._command_manager.schedule(request)
         else:
             logger.error(f"Invalid command type scheduled: '{name}'")
             frontend_logger.error(f"Unknown command: '{name}'")
@@ -744,7 +531,7 @@ class Engine(InterpreterContext):
             self.set_error_state(ex)
             raise
 
-    # code manipulation api
+    # Code manipulation api
     def set_method(self, method: Mdl.Method) -> Literal["merge_method", "set_method"]:
         """ Set new method. This will replace the current method, either by merging in changes in case the method is already
         running or just setting the method otherwise. """
@@ -785,37 +572,27 @@ class Engine(InterpreterContext):
             self.set_error_state(ex)
             raise
 
+    # Cancel/Force commands from user, originating from runlog item - lock should be used here, right?
     def cancel_instruction(self, instance_id: str):
-        if not self.tracking.has_instance_id(instance_id):
-            raise ValueError(f"Cannot cancel instruction {instance_id=}, no runtime record found")
-        else:
-            logger.info(f"Cancel instruction {instance_id=} accepted")
-            record = self.tracking.get_record_by_instance_id(instance_id)
-            assert record is not None
-            command = self.tracking.get_command(instance_id)
-            if command is not None:
-                command.cancel()
-                self.tracking.mark_cancelled(command)
-            else:
-                node = self.tracking.get_known_node_by_id(record.node_id)
-                assert node is not None
-                self.tracking.mark_cancelled(node)
+        """ Cancel command instance and finalize it immidiately """
+        with self._lock:
+            assert self._command_manager is not None
+            self._command_manager.cancel_instruction(instance_id)
 
     def force_instruction(self, instance_id: str):
-        if not self.tracking.has_instance_id(instance_id):
-            logger.error(f"Cannot force instruction {instance_id=}, no runtime record found")
-        else:
-            logger.info(f"Force instruction {instance_id=} accepted")
-            record = self.tracking.get_record_by_instance_id(instance_id)
-            assert record is not None
-            command = self.tracking.get_command(instance_id)
-            if command is not None:
-                command.force()
-                self.tracking.mark_forced(command)
-            else:
-                node = self.tracking.get_known_node_by_id(record.node_id)
-                assert node is not None
-                self.tracking.mark_forced(node)
+        """ Force the command instance """
+        with self._lock:
+            assert self._command_manager is not None
+            self._command_manager.force_instruction(instance_id)
+
+    # Cancel/Finalize originating from Stop/Restart commands, i.e. from the tick commands loop
+    def cancel_all_commands(self, source_command_name: str):
+        assert self._command_manager is not None
+        self._command_manager.cancel_commands(source_command_name, finalize=True)
+
+    # This is unnecessary until we support a cancellation cool-down period
+    # def finalize_all_commands(self, source_command_name: str):
+    #     self._command_manager.finalize_commands(source_command_name)
 
     def get_command_definitions(self) -> list[Mdl.CommandDefinition]:
         """ Return engine command definitions. """
