@@ -5,6 +5,7 @@ import time
 from datetime import UTC
 from logging.handlers import QueueHandler
 from queue import Empty, SimpleQueue
+import decimal
 
 import openpectus.protocol.engine_messages as EM
 import openpectus.protocol.models as Mdl
@@ -33,10 +34,13 @@ def to_model_tag(tag: TagValue) -> Mdl.TagValue:
     if tag.tick_time == 0.0:
         logger.warning(f"Tick time was 0 for tag {tag.name}")
         tag.tick_time = time.time()
+    tag_value = tag.value
+    if isinstance(tag_value, decimal.Decimal):
+        tag_value = float(tag_value)
     return Mdl.TagValue(
         name=tag.name,
         tick_time=tag.tick_time,
-        value=tag.value,
+        value=tag_value,
         value_formatted=tag.value_formatted,
         value_unit=tag.unit,
         direction=tag.direction,
@@ -102,7 +106,8 @@ class EngineMessageBuilder():
         archive = archiver.read_last_run_archive(run_id)
         assert archiver.last_run_file_path is not None
         archive_filename = os.path.basename(archiver.last_run_file_path)
-        return EM.RunStoppedMsg(run_id=run_id, runlog=runlog_msg.runlog, method_state=state, archive=archive, archive_filename=archive_filename)
+        return EM.RunStoppedMsg(run_id=run_id, runlog=runlog_msg.runlog, method_state=state, archive=archive,
+                                archive_filename=archive_filename)
 
     def create_runlog_msg(self, run_id: str) -> EM.RunLogMsg:
         tag_names: list[str] = []
@@ -193,7 +198,7 @@ class EngineMessageBuilder():
         return EM.WebPushNotificationMsg(
             notification=EM.WebPushNotification(
                 title=self.engine.uod.instrument,
-                body=f"Run has been paused.",
+                body="Run has been paused.",
             ),
             topic=EM.NotificationTopic.RUN_PAUSE,
         )
@@ -229,7 +234,7 @@ class EngineMessageBuilder():
         return EM.WebPushNotificationMsg(
             notification=EM.WebPushNotification(
                 title=self.engine.uod.instrument,
-                body=f"Error in method. Run paused.",
+                body="Error in method. Run paused.",
             ),
             topic=EM.NotificationTopic.METHOD_ERROR,
         )
