@@ -9,7 +9,6 @@ from openpectus.lang.exec.events import RunStateChange
 from openpectus.lang.exec.regex import REGEX_DURATION_OPTIONAL, REGEX_TEXT, get_duration_end
 from openpectus.lang.exec.tags import SystemTagName
 from openpectus.engine.engine import Engine
-from openpectus.lang.exec.units import as_float
 
 
 logger = logging.getLogger(__name__)
@@ -60,13 +59,13 @@ class PauseEngineCommand(InternalEngineCommand):
 
     def _run(self):
         duration_end_time : float | None = None
-        time = as_float(self.kvargs.pop("number", ""))
+        time = self.kvargs.pop("number", None)
         if time is not None:
             try:
                 unit = self.kvargs.pop("number_unit")
             except Exception:
                 raise ValueError(f"Argument error. Actual kvargs: {self.kvargs}")
-            duration_end_time = get_duration_end(self.engine._tick_time, time, unit)
+            duration_end_time = get_duration_end(self.engine._tick_time, float(time), unit)
 
         e = self.engine
         e._runstate_paused = True
@@ -79,11 +78,16 @@ class PauseEngineCommand(InternalEngineCommand):
             while self.engine._tick_time < duration_end_time:
                 yield
             logger.debug("Resuming using Unpause")
-            # reusing instance_id is ok
             unpause = UnpauseEngineCommand(self.engine, self._registry)
-            unpause.instance_id = self.instance_id
             unpause._run()
 
+    def cancel(self):
+        super().cancel()
+        logger.debug("Pause cancelled via api. Resuming using Unpause")
+        unpause = UnpauseEngineCommand(self.engine, self._registry)
+        unpause._run()
+        # cancelling hold means aborting wait and completing the command
+        self.set_complete()
 
 @command_argument_none()
 class UnpauseEngineCommand(InternalEngineCommand):
@@ -120,13 +124,13 @@ class HoldEngineCommand(InternalEngineCommand):
 
     def _run(self):
         duration_end_time : float | None = None
-        time = as_float(self.kvargs.pop("number", ""))
+        time = self.kvargs.pop("number", None)
         if time is not None:
             try:
                 unit = self.kvargs.pop("number_unit")
             except Exception:
                 raise ValueError(f"Argument error. Actual kvargs: {self.kvargs}")
-            duration_end_time = get_duration_end(self.engine._tick_time, time, unit)
+            duration_end_time = get_duration_end(self.engine._tick_time, float(time), unit)
 
         e = self.engine
         e._runstate_holding = True
@@ -139,9 +143,15 @@ class HoldEngineCommand(InternalEngineCommand):
                 yield
             logger.debug("Resuming using Unhold")
             unhold = UnholdEngineCommand(self.engine, self._registry)
-            unhold.instance_id = self.instance_id
             unhold._run()
 
+    def cancel(self):
+        super().cancel()
+        logger.debug("Hold cancelled via api. Resuming using Unhold")
+        unhold = UnholdEngineCommand(self.engine, self._registry)
+        unhold._run()
+        self.set_complete()
+
 
 @command_argument_none()
 class UnholdEngineCommand(InternalEngineCommand):
@@ -173,15 +183,17 @@ class StopEngineCommand(InternalEngineCommand):
             self.fail()
         else:
             e._runstate_stopping = True
-            e._cancel_uod_commands()
+            e.cancel_all_commands(source_command_name=self.name)
             yield
-            timeout_at_tick = e._tick_number + CANCEL_TIMEOUT_TICKS
-            while e.uod.has_any_command_instances():
-                if e._tick_number > timeout_at_tick:
-                    logger.warning("Timeout waiting for uod commands to cancel")
-                    break
-                yield
-            e._finalize_uod_commands()
+            # FIXME: Clean up
+            # timeout_at_tick = e._tick_number + CANCEL_TIMEOUT_TICKS
+            # while e.uod.has_any_command_instances():
+            #     if e._tick_number > timeout_at_tick:
+            #         logger.warning("Timeout waiting for uod commands to cancel")
+            #         break
+            #     yield
+
+            # e.finalize_all_commands(source_command_name=self.name)
             e._apply_safe_state()
             e.write_process_image()
 
@@ -220,15 +232,16 @@ class RestartEngineCommand(InternalEngineCommand):
             sys_state.set_value(SystemStateEnum.Restarting, e._tick_time)
 
             e._runstate_stopping = True
-            e._cancel_uod_commands()
+            e.cancel_all_commands(source_command_name=self.name)
             yield
-            timeout_at_tick = e._tick_number + CANCEL_TIMEOUT_TICKS
-            while e.uod.has_any_command_instances():
-                if e._tick_number > timeout_at_tick:
-                    logger.warning("Timed out waiting for uod commands to cancel")
-                    break
-                yield
-            e._finalize_uod_commands()
+            # FIXME: Clean up
+            # timeout_at_tick = e._tick_number + CANCEL_TIMEOUT_TICKS
+            # while e.uod.has_any_command_instances():
+            #     if e._tick_number > timeout_at_tick:
+            #         logger.warning("Timed out waiting for uod commands to cancel")
+            #         break
+            #     yield
+            # e.finalize_all_commands(source_command_name=self.name)
 
             logger.debug("Restarting run - uod commands have completed execution")
             e._runstate_started = False
@@ -241,8 +254,10 @@ class RestartEngineCommand(InternalEngineCommand):
             e.emitter.emit_on_stop()
 
             e.clear_run_id()
+
+            # _stop_interpreter() restarts command_manager
             e._stop_interpreter()
-            logger.info("Restarting run - engine stopped")
+            logger.info(f"Restarting run - engine stopped, tick_number: {e._tick_number}")
 
             yield
 
