@@ -3,6 +3,7 @@ from __future__ import annotations
 import time
 from enum import StrEnum, auto
 from typing import Any, Callable, Iterable, Set
+import decimal
 
 from openpectus.lang.exec.events import EventListener
 from openpectus.lang.exec.units import convert_value_to_unit, is_supported_unit
@@ -39,7 +40,7 @@ class SystemTagName(StrEnum):
         return value in SystemTagName.__members__.values()
 
 
-TagValueType = int | float | str | None
+TagValueType = decimal.Decimal | int | float | str | None
 """ Represents the types of tag values """
 
 TagFormatFunction = Callable[[Any], str]
@@ -188,7 +189,7 @@ class Tag(ChangeSubject, EventListener):
 
     def set_value_and_unit(self, val: TagValueType, unit: str, tick_time: float) -> None:
         """ Set a new value by converting the provided value and unit into the the unit of the tag. """
-        if not isinstance(val, (int, float,)):
+        if not isinstance(val, (int, float, decimal.Decimal)):
             raise ValueError(f"Cannot set unit for a non-numeric value {val} of type {type(val).__name__}")
         if self.unit is None:
             raise ValueError("Cannot change unit on a tag with no unit")
@@ -198,7 +199,7 @@ class Tag(ChangeSubject, EventListener):
     def simulate_value_and_unit(self, val: TagValueType, unit: str, tick_time: float) -> None:
         """ Set a simulated value by converting the provided value and unit into the the unit of the tag. """
         self.simulated = True
-        if not isinstance(val, (int, float,)):
+        if not isinstance(val, (int, float, decimal.Decimal)):
             raise ValueError(f"Cannot set unit for a non-numeric value {val} of type {type(val).__name__}")
         if self.unit is None:
             raise ValueError("Cannot change unit on a tag with no unit")
