@@ -264,7 +264,8 @@ class TestAggregatorRestart(TestUsingConsoleAppRunner):
         engine.stop()
         aggregator.stop()
 
-    @unittest.skipIf(bool(os.environ.get("OPENPECTUS_INTEGRATION_SKIP_SLOW_TESTS")), reason="Skipping slow tests as configured")
+    @unittest.skipIf(bool(os.environ.get("OPENPECTUS_INTEGRATION_SKIP_SLOW_TESTS")),
+                     reason="Skipping slow tests as configured")
     @unittest.skipUnless(sys.platform.lower() == "linux", "This test cannot run correctly on Windows.")
     def test_reconnect_single_heavy_engine(self, port: int = 8719):
         temporary_sqlite_db_filename = tempfile.NamedTemporaryFile(suffix=".sqlite3").name
@@ -355,7 +356,8 @@ class TestAggregatorRestart(TestUsingConsoleAppRunner):
         engine.stop()
         aggregator.stop()
 
-    @unittest.skipIf(bool(os.environ.get("OPENPECTUS_INTEGRATION_SKIP_SLOW_TESTS")), reason="Skipping slow tests as configured")
+    @unittest.skipIf(bool(os.environ.get("OPENPECTUS_INTEGRATION_SKIP_SLOW_TESTS")),
+                     reason="Skipping slow tests as configured")
     @unittest.skipUnless(sys.platform.lower() == "linux", "This test cannot run correctly on Windows.")
     def test_reconnect_multiple_engines(self, port: int = 8720, n_engines=5):
         temporary_sqlite_db_filename = tempfile.NamedTemporaryFile(suffix=".sqlite3").name
@@ -430,7 +432,8 @@ class TestAggregatorRestart(TestUsingConsoleAppRunner):
                 command = ExecutableCommand(command=command_name, source=CommandSource.UNIT_BUTTON)
                 responses = []
                 for process_unit in process_units:
-                    url = f"http://127.0.0.1:{port}/api/process_unit/{urllib.parse.quote(process_unit.id)}/execute_control_button_command"
+                    unit_id = urllib.parse.quote(process_unit.id)
+                    url = f"http://127.0.0.1:{port}/api/process_unit/{unit_id}/execute_control_button_command"
                     responses.append(client.post(url, json=command.model_dump(), timeout=10))
                 await asyncio.gather(*responses)
 
