@@ -17,6 +17,7 @@ from openpectus.lang.exec.commands import CommandCollection, Command
 from openpectus.lang.exec.tags import TagValueCollection, TagValue
 from openpectus.lang.exec.argument_specification import ArgSpec
 from openpectus.lang.exec.uod import RegexNamedArgumentParser
+from openpectus.lang.model.pprogramformatter import print_program
 from openpectus.test.engine.utility_methods import build_program
 
 
@@ -262,6 +263,45 @@ Block: A
         self.assertEqual(item.range.start.line, 2)
         self.assertEqual(item.range.end.line, 3)
 
+    def test_block_starting_w_comment(self):
+        program = build_program("""
+Block: A
+    # Comment
+    End block
+""")
+        print_program(program, show_line_numbers=True, show_errors=True, show_blanks=True)
+        analyzer = IndentationCheckAnalyzer()
+        analyzer.analyze(program)
+        for item in analyzer.items:
+            print(item)
+        self.assertEqual(0, len(analyzer.items))
+
+    def test_block_starting_w_multiple_comments(self):
+        program = build_program("""
+Block: A
+    # Comment line 1
+    # Comment line 2
+    End block
+""")
+        analyzer = IndentationCheckAnalyzer()
+        analyzer.analyze(program)
+        for item in analyzer.items:
+            print(item)
+        self.assertEqual(0, len(analyzer.items))
+
+    def test_block_w_excessive_indentation(self):
+        program = build_program("""
+Block: A
+        Block: B
+        End block
+    End block
+""")
+        analyzer = IndentationCheckAnalyzer()
+        analyzer.analyze(program)
+        for item in analyzer.items:
+            print(item)
+        assert len(analyzer.items) > 0
+
 
 class ThresholdCheckAnalyzerTest(unittest.TestCase):
     def test_correct_order(self):
@@ -802,7 +842,8 @@ Food: bar
         program = build_program("""
 Foo
 """)
-        cmds = CommandCollection().with_cmd(Command("Foo", arg_parser=RegexNamedArgumentParser(regex=ArgSpec.NoArgsInstance.regex)))
+        cmds = CommandCollection().with_cmd(
+            Command("Foo", arg_parser=RegexNamedArgumentParser(regex=ArgSpec.NoArgsInstance.regex)))
         analyzer = CommandCheckAnalyzer(cmds)
         analyzer.analyze(program)
         self.assertEqual(0, len(analyzer.items))
@@ -811,7 +852,8 @@ Foo
         program = build_program("""
 Foo: bar
 """)
-        cmds = CommandCollection().with_cmd(Command("Foo", arg_parser=RegexNamedArgumentParser(regex=ArgSpec.NoArgsInstance.regex)))
+        cmds = CommandCollection().with_cmd(
+            Command("Foo", arg_parser=RegexNamedArgumentParser(regex=ArgSpec.NoArgsInstance.regex)))
         analyzer = CommandCheckAnalyzer(cmds)
         analyzer.analyze(program)
         self.assertEqual(1, len(analyzer.items))
